package it.sauronsoftware.kickstart;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;

/**
 * La finestra con le informazioni sul programma.
 * 
 * @author Carlo Pelliccia
 */
public class AboutDialog extends JDialog {

	private static final long serialVersionUID = 1L;

	/**
	 * Costruisce la finestra di dialogo con le informazioni sul programma.
	 * 
	 * @param owner
	 *            Il proprietario della finestra.
	 */
	public AboutDialog(Kickstart owner) {
		super(owner, true);
		LanguagePack pack = LanguagePack.getInstance();
		setTitle(pack.getMessage("ABOUT_DIALOG_TITLE"));
		initializeGUI();
		pack();
		setResizable(false);
		setLocationRelativeTo(owner);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
	}

	/**
	 * Inizializza la GUI.
	 */
	private void initializeGUI() {
		GridBagConstraints c00 = new GridBagConstraints();
		c00.gridy = 0;
		c00.gridx = 0;
		c00.anchor = GridBagConstraints.CENTER;
		c00.fill = GridBagConstraints.NONE;
		GridBagConstraints c01 = new GridBagConstraints();
		c00.gridy = 0;
		c00.gridx = 1;
		c00.anchor = GridBagConstraints.CENTER;
		c00.fill = GridBagConstraints.NONE;
		GridBagConstraints c10 = new GridBagConstraints();
		c10.gridy = 1;
		c10.gridx = 0;
		c10.anchor = GridBagConstraints.CENTER;
		c10.fill = GridBagConstraints.NONE;
		c10.gridwidth = GridBagConstraints.REMAINDER;
		GridBagConstraints c20 = new GridBagConstraints();
		c20.gridy = 2;
		c20.gridx = 0;
		c20.anchor = GridBagConstraints.CENTER;
		c20.fill = GridBagConstraints.NONE;
		c20.gridwidth = GridBagConstraints.REMAINDER;
		JPanel contentPane = new JPanel();
		contentPane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		contentPane.setLayout(new GridBagLayout());
		contentPane.add(getInfoPanel(), c00);
		contentPane.add(getLinkPanel(), c01);
		contentPane.add(getDonationPanel(), c10);
		contentPane.add(getClosePanel(), c20);
		setContentPane(contentPane);
	}

	/**
	 * Prepara e restituisce il pannello con le informazioni sul programma.
	 * 
	 * @return Il pannello con le informazioni sul programma.
	 */
	private JPanel getInfoPanel() {
		LanguagePack pack = LanguagePack.getInstance();
		JLabel title = new JLabel(pack.getMessage("ABOUT_DIALOG_INFO_TITLE"));
		JLabel credits = new JLabel(pack
				.getMessage("ABOUT_DIALOG_INFO_CREDITS"));
		JLabel copyright = new JLabel(pack
				.getMessage("ABOUT_DIALOG_INFO_COPYRIGHT"));
		title.setFont(title.getFont().deriveFont(
				title.getFont().getSize2D() * 1.25F).deriveFont(Font.BOLD));
		JPanel infoPanel = new JPanel(new GridLayout(3, 1, 3, 3));
		infoPanel.add(title);
		infoPanel.add(credits);
		infoPanel.add(copyright);
		infoPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		return infoPanel;
	}

	/**
	 * Prepara e restituisce il pannello con i collegamenti web.
	 * 
	 * @return Il pannello con i collegamenti web.
	 */
	private JPanel getLinkPanel() {
		GridBagConstraints c00 = new GridBagConstraints();
		c00.gridy = 0;
		c00.gridx = 0;
		c00.anchor = GridBagConstraints.CENTER;
		c00.fill = GridBagConstraints.HORIZONTAL;
		c00.insets = new Insets(5, 5, 5, 5);
		GridBagConstraints c10 = new GridBagConstraints();
		c10.gridy = 1;
		c10.gridx = 0;
		c10.anchor = GridBagConstraints.CENTER;
		c10.fill = GridBagConstraints.HORIZONTAL;
		c10.insets = new Insets(5, 5, 5, 5);
		final LanguagePack pack = LanguagePack.getInstance();
		JButton project = new JButton(pack
				.getMessage("ABOUT_DIALOG_LINK_KICKSTART"));
		project.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				openLink(pack.getMessage("ABOUT_DIALOG_LINK_KICKSTART_URL"));
			}
		});
		JButton author = new JButton(pack
				.getMessage("ABOUT_DIALOG_LINK_AUTHOR"));
		author.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				openLink(pack.getMessage("ABOUT_DIALOG_LINK_AUTHOR_URL"));
			}
		});
		JPanel linkPanel = new JPanel();
		linkPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		linkPanel.setLayout(new GridBagLayout());
		linkPanel.add(project, c00);
		linkPanel.add(author, c10);
		return linkPanel;
	}

	/**
	 * Prepara e restituisce il pannello con il link per le donazioni.
	 * 
	 * @return Il pannello con il link per le donazioni.
	 */
	private JPanel getDonationPanel() {
		final LanguagePack pack = LanguagePack.getInstance();
		JLabel donationLabel = new JLabel(pack
				.getMessage("ABOUT_DIALOG_DONATION_LABEL"));
		JButton donationButton = new JButton(pack
				.getMessage("ABOUT_DIALOG_DONATION_BUTTON_LABEL"));
		donationButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				openLink(pack.getMessage("ABOUT_DIALOG_DONATION_BUTTON_URL"));
			}
		});
		JPanel donationPanel = new JPanel();
		donationPanel.setLayout(new BorderLayout(5, 5));
		donationPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		donationPanel.add(donationLabel, BorderLayout.CENTER);
		donationPanel.add(donationButton, BorderLayout.EAST);
		return donationPanel;
	}

	/**
	 * Prepara e restituisce il pannello per la chiusura della finestra.
	 * 
	 * @return Il pannello per la chiusura della finestra.
	 */
	private JPanel getClosePanel() {
		final LanguagePack pack = LanguagePack.getInstance();
		JButton closeButton = new JButton(pack
				.getMessage("ABOUT_DIALOG_CLOSE_LABEL"));
		closeButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dispose();
			}
		});
		JPanel closePanel = new JPanel();
		closePanel.setLayout(new FlowLayout(FlowLayout.CENTER));
		closePanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		closePanel.add(closeButton, BorderLayout.EAST);
		return closePanel;
	}

	/**
	 * Apre un link nel browser.
	 * 
	 * @param url
	 *            L'indirizzo da aprire.
	 */
	private void openLink(String url) {
		Runtime runtime = Runtime.getRuntime();
		try {
			runtime.exec("rundll32 url.dll,FileProtocolHandler " + url);
		} catch (IOException e) {
			;
		}
	}

}
