package it.sauronsoftware.kickstart;

import it.sauronsoftware.kickstart.filters.EXEFileFilter;
import it.sauronsoftware.kickstart.filters.IconFileFilter;
import it.sauronsoftware.kickstart.filters.JARFileFilter;
import it.sauronsoftware.kickstart.filters.ProjectFileFilter;
import it.sauronsoftware.kickstart.generator.EXEGenerator;
import it.sauronsoftware.kickstart.project.Project;
import it.sauronsoftware.kickstart.util.ReplacementMap;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.UIManager;

/**
 * La classe principale del programma, che  costituita dal frame.
 * 
 * @author Carlo Pelliccia
 */
public class Kickstart extends JFrame {

	private static final long serialVersionUID = 2L;

	/**
	 * Il progetto corrente.
	 */
	private Project project = null;

	/**
	 * Il pannello con il logo del software.
	 */
	private JPanel logoPanel = null;

	/**
	 * Il pannello con la parte della form per l'impostazione del nome
	 * dell'applicazione.
	 */
	private JPanel applicationNamePanel = null;

	/**
	 * Il pannello con i widget per impostare gli argomenti per la JVM.
	 */
	private JPanel jvmParametersPanel = null;

	/**
	 * Il pannello con i widget per la selezione dell'icona da associare al
	 * programma.
	 */
	private JPanel iconSelectionPanel = null;

	/**
	 * Il pannello per la selezione del JAR.
	 */
	private JPanel jarSelectionPanel = null;

	/**
	 * Il pannello per la selezione dell'EXE.
	 */
	private JPanel exeSelectionPanel = null;

	/**
	 * Il pannello per la selezione della tipologia dell'applicazione
	 * (finestre/console).
	 */
	private JPanel applicationTypePanel = null;

	/**
	 * Il pannello con il pulsante di azione per la generazione del file EXE.
	 */
	private JPanel actionPanel = null;

	/**
	 * Il campo per il nome dell'applicazione.
	 */
	private JTextField applicationNameField = null;

	/**
	 * Il campo per gli argomenti alla JVM.
	 */
	private JTextField jvmParametersField = null;

	/**
	 * Il campo con il path del JAR.
	 */
	private JTextField jarSelectionField = null;

	/**
	 * Il campo con il path dell'EXE.
	 */
	private JTextField exeSelectionField = null;

	/**
	 * Il campo con il path dell'icona.
	 */
	private JTextField iconSelectionField = null;

	/**
	 * L'opzione "applicazione con GUI".
	 */
	private JRadioButton guiRadioButton = null;

	/**
	 * L'opzione "applicazione per console".
	 */
	private JRadioButton consoleRadioButton = null;

	/**
	 * Costruisce l'applicazione ed il suo frame.
	 */
	public Kickstart() {
		super(LanguagePack.getInstance().getMessage("MAIN_WINDOW_TITLE"));
		initializeMenu();
		initializeGUI();
		pack();
		setResizable(false);
		locateFrameOnScreen();
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				dispose();
			}
		});
	}

	/**
	 * Posiziona la finestra sullo schermo.
	 */
	private void locateFrameOnScreen() {
		Preferences preferences = Preferences.getInstance();
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension frameSize = getSize();
		int defLeft = (screenSize.width - frameSize.width) / 2;
		int defTop = (screenSize.height - frameSize.height) / 2;
		int left = preferences.getInt(getClass().getName() + ".left", defLeft);
		int top = preferences.getInt(getClass().getName() + ".top", defTop);
		if (left < 0) {
			left = 0;
		}
		if (top < 0) {
			top = 0;
		}
		if (left > screenSize.width - frameSize.width) {
			left = screenSize.width - frameSize.width;
		}
		if (top > screenSize.height - frameSize.height) {
			top = screenSize.height - frameSize.height;
		}
		setLocation(left, top);
	}

	/**
	 * Imposta le preferenze e chiude l'applicazione.
	 */
	public void dispose() {
		if (checkProjectDiscard()) {
			// Salva la posizione della finestra.
			Point location = getLocation();
			Preferences preferences = Preferences.getInstance();
			preferences.putInt(getClass().getName() + ".left", location.x);
			preferences.putInt(getClass().getName() + ".top", location.y);
			// Fa il dispose a livello superiore.
			super.dispose();
			// Termina il programma.
			System.exit(0);
		}
	}

	/**
	 * Inizializza i componenti grafici dell'applicazione.
	 */
	private void initializeGUI() {
		GridBagConstraints row0 = new GridBagConstraints();
		row0.gridy = 0;
		row0.gridx = 0;
		row0.weightx = 1.0;
		row0.fill = GridBagConstraints.HORIZONTAL;
		GridBagConstraints row1 = new GridBagConstraints();
		row1.gridy = 1;
		row1.gridx = 0;
		row1.weightx = 1.0;
		row1.fill = GridBagConstraints.HORIZONTAL;
		GridBagConstraints row2 = new GridBagConstraints();
		row2.gridy = 2;
		row2.gridx = 0;
		row2.weightx = 1.0;
		row2.fill = GridBagConstraints.HORIZONTAL;
		GridBagConstraints row3 = new GridBagConstraints();
		row3.gridy = 3;
		row3.gridx = 0;
		row3.weightx = 1.0;
		row3.fill = GridBagConstraints.HORIZONTAL;
		GridBagConstraints row4 = new GridBagConstraints();
		row4.gridy = 4;
		row4.gridx = 0;
		row4.weightx = 1.0;
		row4.fill = GridBagConstraints.HORIZONTAL;
		GridBagConstraints row5 = new GridBagConstraints();
		row5.gridy = 5;
		row5.gridx = 0;
		row5.weightx = 1.0;
		row5.fill = GridBagConstraints.HORIZONTAL;
		GridBagConstraints row6 = new GridBagConstraints();
		row6.gridy = 6;
		row6.gridx = 0;
		row6.weightx = 1.0;
		row6.fill = GridBagConstraints.HORIZONTAL;
		GridBagConstraints row7 = new GridBagConstraints();
		row7.gridy = 7;
		row7.gridx = 0;
		row7.weightx = 1.0;
		row7.fill = GridBagConstraints.HORIZONTAL;
		JPanel contentPane = new JPanel();
		contentPane.setLayout(new GridBagLayout());
		contentPane.add(getLogoPanel(), row0);
		contentPane.add(getApplicationNamePanel(), row1);
		contentPane.add(getApplicationTypePanel(), row2);
		contentPane.add(getJarSelectionPanel(), row3);
		contentPane.add(getExeSelectionPanel(), row4);
		contentPane.add(getIconSelectionPanel(), row5);
		contentPane.add(getJvmParametersPanel(), row6);
		contentPane.add(getActionPanel(), row7);
		setContentPane(contentPane);
	}

	/**
	 * Inizializza il menu nella finestra.
	 */
	private void initializeMenu() {
		LanguagePack pack = LanguagePack.getInstance();
		JMenuBar menuBar = new JMenuBar();
		// Progetto.
		JMenu menu1 = new JMenu(pack.getMessage("MAIN_WINDOW_MENU1"));
		// ?.
		JMenu menu2 = new JMenu(pack.getMessage("MAIN_WINDOW_MENU2"));
		// Nuovo.
		JMenuItem item1_1 = new JMenuItem(pack
				.getMessage("MAIN_WINDOW_MENU1_1"));
		item1_1.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				newProject();
			}
		});
		// Apri.
		JMenuItem item1_2 = new JMenuItem(pack
				.getMessage("MAIN_WINDOW_MENU1_2"));
		item1_2.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				openProject();
			}
		});
		// Salva.
		JMenuItem item1_3 = new JMenuItem(pack
				.getMessage("MAIN_WINDOW_MENU1_3"));
		item1_3.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saveProject(false);
			}
		});
		// Salva con nome.
		JMenuItem item1_4 = new JMenuItem(pack
				.getMessage("MAIN_WINDOW_MENU1_4"));
		item1_4.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saveProject(true);
			}
		});
		// Esci.
		JMenuItem item1_5 = new JMenuItem(pack
				.getMessage("MAIN_WINDOW_MENU1_5"));
		item1_5.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dispose();
			}
		});
		// Informazioni.
		JMenuItem item2_1 = new JMenuItem(pack
				.getMessage("MAIN_WINDOW_MENU2_1"));
		item2_1.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				showAboutDialog();
			}
		});
		// Assembla.
		menu1.add(item1_1);
		menu1.add(item1_2);
		menu1.addSeparator();
		menu1.add(item1_3);
		menu1.add(item1_4);
		menu1.addSeparator();
		menu1.add(item1_5);
		menu2.add(item2_1);
		menuBar.add(menu1);
		menuBar.add(menu2);
		// Imposta.
		setJMenuBar(menuBar);
	}

	/**
	 * Restituisce il pannello con il logo del software.
	 * 
	 * @return Il pannello con il logo del software.
	 */
	private JPanel getLogoPanel() {
		if (logoPanel == null) {
			logoPanel = new JPanel();
			logoPanel.setLayout(new BorderLayout(0, 0));
			logoPanel.add(new JLabel(IconLoader.load("kickstart.png")),
					BorderLayout.NORTH);
		}
		return logoPanel;
	}

	/**
	 * Restituisce il pannello con la parte della form per l'impostazione del
	 * nome dell'applicazione.
	 * 
	 * @return Il pannello con la parte della form per l'impostazione del nome
	 *         dell'applicazione.
	 */
	private JPanel getApplicationNamePanel() {
		if (applicationNamePanel == null) {
			applicationNameField = new JTextField();
			LanguagePack pack = LanguagePack.getInstance();
			GridBagConstraints row0 = new GridBagConstraints();
			row0.gridy = 0;
			row0.gridx = 0;
			row0.weightx = 1.0;
			row0.fill = GridBagConstraints.HORIZONTAL;
			GridBagConstraints row1 = new GridBagConstraints();
			row1.gridy = 1;
			row1.gridx = 0;
			row1.weightx = 1.0;
			row1.fill = GridBagConstraints.HORIZONTAL;
			applicationNamePanel = new JPanel();
			applicationNamePanel.setBorder(BorderFactory.createEmptyBorder(5,
					5, 5, 5));
			applicationNamePanel.setLayout(new GridBagLayout());
			applicationNamePanel.add(new JLabel(pack
					.getMessage("MAIN_WINDOW_LABEL1")), row0);
			applicationNamePanel.add(applicationNameField, row1);
		}
		return applicationNamePanel;
	}

	/**
	 * Restituisce il pannello con la parte della form per l'impostazione dei
	 * parametri della JVM.
	 * 
	 * @return Il pannello con la parte della form per l'impostazione dei
	 *         parametri della JVM.
	 */
	private JPanel getJvmParametersPanel() {
		if (jvmParametersPanel == null) {
			jvmParametersField = new JTextField();
			LanguagePack pack = LanguagePack.getInstance();
			GridBagConstraints row0 = new GridBagConstraints();
			row0.gridy = 0;
			row0.gridx = 0;
			row0.weightx = 1.0;
			row0.fill = GridBagConstraints.HORIZONTAL;
			GridBagConstraints row1 = new GridBagConstraints();
			row1.gridy = 1;
			row1.gridx = 0;
			row1.weightx = 1.0;
			row1.fill = GridBagConstraints.HORIZONTAL;
			jvmParametersPanel = new JPanel();
			jvmParametersPanel.setBorder(BorderFactory.createEmptyBorder(5, 5,
					5, 5));
			jvmParametersPanel.setLayout(new GridBagLayout());
			jvmParametersPanel.add(new JLabel(pack
					.getMessage("MAIN_WINDOW_LABEL12")), row0);
			jvmParametersPanel.add(jvmParametersField, row1);
		}
		return jvmParametersPanel;
	}

	/**
	 * Restituisce il pannello con i widget per la selezione dell'icona da
	 * associare al programma.
	 * 
	 * @return Il pannello con i widget per la selezione dell'icona da associare
	 *         al programma.
	 */
	private JPanel getIconSelectionPanel() {
		if (iconSelectionPanel == null) {
			final Kickstart myself = this;
			final LanguagePack pack = LanguagePack.getInstance();
			iconSelectionField = new JTextField();
			JButton browseButton = new JButton();
			browseButton.setIcon(IconLoader.load("browse.png"));
			browseButton.setToolTipText(pack.getMessage("MAIN_WINDOW_LABEL3"));
			browseButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					JFileChooser chooser = new JFileChooser();
					chooser.setFileFilter(new IconFileFilter());
					chooser.setMultiSelectionEnabled(false);
					chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
					chooser.setDialogTitle(pack
							.getMessage("FILECHOOSER_ICON_TITLE"));
					chooser.setApproveButtonText(pack
							.getMessage("FILECHOOSER_ICON_APPROVE"));
					String selectedPath = iconSelectionField.getText().trim();
					if (selectedPath.length() > 0) {
						File selectedFile = new File(selectedPath);
						chooser.setCurrentDirectory(selectedFile
								.getParentFile());
						chooser.setSelectedFile(selectedFile);
					} else {
						Preferences preferences = Preferences.getInstance();
						String dirPath = preferences.getString(getClass()
								.getName()
								+ ".icon.directory");
						if (dirPath != null) {
							chooser.setCurrentDirectory(new File(dirPath));
						} else {
							chooser.setCurrentDirectory(null);
						}
					}
					int c = chooser.showOpenDialog(myself);
					if (c == JFileChooser.APPROVE_OPTION) {
						File selectedFile = chooser.getSelectedFile();
						if (selectedFile != null) {
							iconSelectionField.setText(selectedFile
									.getAbsolutePath());
							File selectedDir = selectedFile.getParentFile();
							Preferences preferences = Preferences.getInstance();
							preferences.putString(getClass().getName()
									+ ".icon.directory", selectedDir
									.getAbsolutePath());
						}
					}
				}
			});
			GridBagConstraints row00 = new GridBagConstraints();
			row00.gridy = 0;
			row00.gridx = 0;
			row00.weightx = 1.0;
			row00.fill = GridBagConstraints.HORIZONTAL;
			GridBagConstraints row10 = new GridBagConstraints();
			row10.gridy = 1;
			row10.gridx = 0;
			row10.weightx = 0.99;
			row10.fill = GridBagConstraints.HORIZONTAL;
			GridBagConstraints row11 = new GridBagConstraints();
			row11.gridy = 1;
			row11.gridx = 1;
			row11.weightx = 0.01;
			row11.insets = new Insets(0, 3, 0, 0);
			row11.fill = GridBagConstraints.NONE;
			iconSelectionPanel = new JPanel();
			iconSelectionPanel.setBorder(BorderFactory.createEmptyBorder(5, 5,
					5, 5));
			iconSelectionPanel.setLayout(new GridBagLayout());
			iconSelectionPanel.add(new JLabel(pack
					.getMessage("MAIN_WINDOW_LABEL2")), row00);
			iconSelectionPanel.add(iconSelectionField, row10);
			iconSelectionPanel.add(browseButton, row11);
		}
		return iconSelectionPanel;
	}

	/**
	 * Restituisce il pannello per la selezione del JAR.
	 * 
	 * @return Il pannello per la selezione del JAR.
	 */
	private JPanel getJarSelectionPanel() {
		if (jarSelectionPanel == null) {
			final Kickstart myself = this;
			final LanguagePack pack = LanguagePack.getInstance();
			jarSelectionField = new JTextField();
			JButton browseButton = new JButton();
			browseButton.setIcon(IconLoader.load("browse.png"));
			browseButton.setToolTipText(pack.getMessage("MAIN_WINDOW_LABEL5"));
			browseButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					JFileChooser chooser = new JFileChooser();
					chooser.setFileFilter(new JARFileFilter());
					chooser.setMultiSelectionEnabled(false);
					chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
					chooser.setDialogTitle(pack
							.getMessage("FILECHOOSER_JAR_TITLE"));
					chooser.setApproveButtonText(pack
							.getMessage("FILECHOOSER_JAR_APPROVE"));
					String selectedPath = jarSelectionField.getText().trim();
					if (selectedPath.length() > 0) {
						File selectedFile = new File(selectedPath);
						chooser.setCurrentDirectory(selectedFile
								.getParentFile());
						chooser.setSelectedFile(selectedFile);
					} else {
						Preferences preferences = Preferences.getInstance();
						String dirPath = preferences.getString(getClass()
								.getName()
								+ ".jar.directory");
						if (dirPath != null) {
							chooser.setCurrentDirectory(new File(dirPath));
						} else {
							chooser.setCurrentDirectory(null);
						}
					}
					int c = chooser.showOpenDialog(myself);
					if (c == JFileChooser.APPROVE_OPTION) {
						File selectedFile = chooser.getSelectedFile();
						if (selectedFile != null) {
							jarSelectionField.setText(selectedFile
									.getAbsolutePath());
							File selectedDir = selectedFile.getParentFile();
							Preferences preferences = Preferences.getInstance();
							preferences.putString(getClass().getName()
									+ ".jar.directory", selectedDir
									.getAbsolutePath());
						}
					}
				}
			});
			GridBagConstraints row00 = new GridBagConstraints();
			row00.gridy = 0;
			row00.gridx = 0;
			row00.weightx = 1.0;
			row00.fill = GridBagConstraints.HORIZONTAL;
			GridBagConstraints row10 = new GridBagConstraints();
			row10.gridy = 1;
			row10.gridx = 0;
			row10.weightx = 0.99;
			row10.fill = GridBagConstraints.HORIZONTAL;
			GridBagConstraints row11 = new GridBagConstraints();
			row11.gridy = 1;
			row11.gridx = 1;
			row11.weightx = 0.01;
			row11.insets = new Insets(0, 3, 0, 0);
			row11.fill = GridBagConstraints.NONE;
			jarSelectionPanel = new JPanel();
			jarSelectionPanel.setBorder(BorderFactory.createEmptyBorder(5, 5,
					5, 5));
			jarSelectionPanel.setLayout(new GridBagLayout());
			jarSelectionPanel.add(new JLabel(pack
					.getMessage("MAIN_WINDOW_LABEL4")), row00);
			jarSelectionPanel.add(jarSelectionField, row10);
			jarSelectionPanel.add(browseButton, row11);
		}
		return jarSelectionPanel;
	}

	/**
	 * Restituisce il pannello per la selezione dell'EXE.
	 * 
	 * @return Il pannello per la selezione dell'EXE.
	 */
	private JPanel getExeSelectionPanel() {
		if (exeSelectionPanel == null) {
			final Kickstart myself = this;
			final LanguagePack pack = LanguagePack.getInstance();
			exeSelectionField = new JTextField();
			JButton browseButton = new JButton();
			browseButton.setIcon(IconLoader.load("browse.png"));
			browseButton.setToolTipText(pack.getMessage("MAIN_WINDOW_LABEL7"));
			browseButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					JFileChooser chooser = new JFileChooser();
					chooser.setFileFilter(new EXEFileFilter());
					chooser.setMultiSelectionEnabled(false);
					chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
					chooser.setDialogTitle(pack
							.getMessage("FILECHOOSER_EXE_TITLE"));
					chooser.setApproveButtonText(pack
							.getMessage("FILECHOOSER_EXE_APPROVE"));
					String selectedPath = exeSelectionField.getText().trim();
					if (selectedPath.length() > 0) {
						File selectedFile = new File(selectedPath);
						chooser.setCurrentDirectory(selectedFile
								.getParentFile());
						chooser.setSelectedFile(selectedFile);
					} else {
						Preferences preferences = Preferences.getInstance();
						String dirPath = preferences.getString(getClass()
								.getName()
								+ ".exe.directory");
						if (dirPath != null) {
							chooser.setCurrentDirectory(new File(dirPath));
						} else {
							chooser.setCurrentDirectory(null);
						}
					}
					int c = chooser.showOpenDialog(myself);
					if (c == JFileChooser.APPROVE_OPTION) {
						File selectedFile = chooser.getSelectedFile();
						if (selectedFile != null) {
							exeSelectionField.setText(selectedFile
									.getAbsolutePath());
							File selectedDir = selectedFile.getParentFile();
							Preferences preferences = Preferences.getInstance();
							preferences.putString(getClass().getName()
									+ ".exe.directory", selectedDir
									.getAbsolutePath());
						}
					}
				}
			});
			GridBagConstraints row00 = new GridBagConstraints();
			row00.gridy = 0;
			row00.gridx = 0;
			row00.weightx = 1.0;
			row00.fill = GridBagConstraints.HORIZONTAL;
			GridBagConstraints row10 = new GridBagConstraints();
			row10.gridy = 1;
			row10.gridx = 0;
			row10.weightx = 0.99;
			row10.fill = GridBagConstraints.HORIZONTAL;
			GridBagConstraints row11 = new GridBagConstraints();
			row11.gridy = 1;
			row11.gridx = 1;
			row11.weightx = 0.01;
			row11.insets = new Insets(0, 3, 0, 0);
			row11.fill = GridBagConstraints.NONE;
			exeSelectionPanel = new JPanel();
			exeSelectionPanel.setBorder(BorderFactory.createEmptyBorder(5, 5,
					5, 5));
			exeSelectionPanel.setLayout(new GridBagLayout());
			exeSelectionPanel.add(new JLabel(pack
					.getMessage("MAIN_WINDOW_LABEL6")), row00);
			exeSelectionPanel.add(exeSelectionField, row10);
			exeSelectionPanel.add(browseButton, row11);
		}
		return exeSelectionPanel;
	}

	/**
	 * Restituisce il pannello per la selezione della tipologia
	 * dell'applicazione (finestre/console).
	 * 
	 * @return Il pannello per la selezione della tipologia dell'applicazione
	 *         (finestre/console).
	 */
	private JPanel getApplicationTypePanel() {
		if (applicationTypePanel == null) {
			LanguagePack pack = LanguagePack.getInstance();
			guiRadioButton = new JRadioButton(pack
					.getMessage("MAIN_WINDOW_LABEL9"));
			consoleRadioButton = new JRadioButton(pack
					.getMessage("MAIN_WINDOW_LABEL10"));
			ButtonGroup buttonGroup = new ButtonGroup();
			buttonGroup.add(guiRadioButton);
			buttonGroup.add(consoleRadioButton);
			guiRadioButton.setSelected(true);
			GridBagConstraints row00 = new GridBagConstraints();
			row00.gridy = 0;
			row00.gridx = 0;
			row00.weightx = 1.0;
			row00.gridwidth = 2;
			row00.fill = GridBagConstraints.HORIZONTAL;
			GridBagConstraints row10 = new GridBagConstraints();
			row10.gridy = 1;
			row10.gridx = 0;
			row10.weightx = 0.5;
			row10.fill = GridBagConstraints.NONE;
			GridBagConstraints row11 = new GridBagConstraints();
			row11.gridy = 1;
			row11.gridx = 1;
			row11.weightx = 0.5;
			row11.fill = GridBagConstraints.NONE;
			applicationTypePanel = new JPanel();
			applicationTypePanel.setBorder(BorderFactory.createEmptyBorder(5,
					5, 5, 5));
			applicationTypePanel.setLayout(new GridBagLayout());
			applicationTypePanel.add(new JLabel(pack
					.getMessage("MAIN_WINDOW_LABEL8")), row00);
			applicationTypePanel.add(guiRadioButton, row10);
			applicationTypePanel.add(consoleRadioButton, row11);
		}
		return applicationTypePanel;
	}

	/**
	 * Restituisce il pannello con il pulsante di azione per la generazione del
	 * file EXE.
	 * 
	 * @return Il pannello con il pulsante di azione per la generazione del file
	 *         EXE.
	 */
	private JPanel getActionPanel() {
		if (actionPanel == null) {
			LanguagePack pack = LanguagePack.getInstance();
			JButton button = new JButton();
			button.setText(pack.getMessage("MAIN_WINDOW_LABEL11"));
			button.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					generateEXE();
				}
			});
			actionPanel = new JPanel();
			actionPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
			actionPanel.setLayout(new FlowLayout(FlowLayout.CENTER));
			actionPanel.add(button);
		}
		return actionPanel;
	}

	/**
	 * Valida i campi e lancia la procedura di produzione dell'EXE.
	 */
	private void generateEXE() {
		LanguagePack pack = LanguagePack.getInstance();
		String applicationName = applicationNameField.getText().trim();
		int applicationType = guiRadioButton.isSelected() ? EXEGenerator.TYPE_GUI
				: EXEGenerator.TYPE_CONSOLE;
		String jarPath = jarSelectionField.getText().trim();
		String exePath = exeSelectionField.getText().trim();
		String iconPath = iconSelectionField.getText().trim();
		String jvmParameters = jvmParametersField.getText().trim();
		File jarFile = null;
		File exeFile = null;
		File iconFile = null;
		URL iconURL = null;
		// Prima validazione: sintattica.
		if (applicationName.length() == 0) {
			JOptionPane.showMessageDialog(this, pack
					.getMessage("MAIN_WINDOW_ERROR1_MESSAGE"), pack
					.getMessage("MAIN_WINDOW_ERROR1_TITLE"),
					JOptionPane.ERROR_MESSAGE);
			return;
		}
		if (jarPath.length() == 0) {
			JOptionPane.showMessageDialog(this, pack
					.getMessage("MAIN_WINDOW_ERROR2_MESSAGE"), pack
					.getMessage("MAIN_WINDOW_ERROR2_TITLE"),
					JOptionPane.ERROR_MESSAGE);
			return;
		} else {
			jarFile = new File(jarPath);
		}
		if (exePath.length() == 0) {
			String name = jarFile.getName();
			int i = name.lastIndexOf('.');
			if (i > -1) {
				name = name.substring(0, i);
			}
			name += ".exe";
			exeFile = new File(jarFile.getParentFile(), name);
		} else {
			exeFile = new File(exePath);
		}
		if (iconPath.length() == 0) {
			iconFile = null;
		} else {
			iconFile = new File(iconPath);
		}
		// Validazione "semantica".
		if (!jarFile.exists()) {
			JOptionPane.showMessageDialog(this, pack
					.getMessage("MAIN_WINDOW_ERROR2_MESSAGE"), pack
					.getMessage("MAIN_WINDOW_ERROR2_TITLE"),
					JOptionPane.ERROR_MESSAGE);
			return;
		}
		if (iconFile != null) {
			if (!iconFile.exists()) {
				JOptionPane.showMessageDialog(this, pack
						.getMessage("MAIN_WINDOW_ERROR3_MESSAGE"), pack
						.getMessage("MAIN_WINDOW_ERROR3_TITLE"),
						JOptionPane.ERROR_MESSAGE);
				return;
			} else {
				try {
					iconURL = iconFile.toURI().toURL();
				} catch (MalformedURLException e) {
					;
				}
			}
		}
		if (exeFile.exists()) {
			ReplacementMap map = new ReplacementMap();
			map.addReplacement("%FILENAME%", exeFile.getName());
			int res = JOptionPane.showConfirmDialog(this, pack.getMessage(
					"MAIN_WINDOW_CONFIRM1_MESSAGE", map), pack
					.getMessage("MAIN_WINDOW_CONFIRM1_TITLE"),
					JOptionPane.YES_NO_OPTION);
			if (res != JOptionPane.YES_OPTION) {
				return;
			}
		}
		// Avvia la routine di generazione.
		EXEGenerator generator = new EXEGenerator(applicationName,
				applicationType, jvmParameters, jarFile, exeFile, iconURL);
		GenerationDialog dialog = new GenerationDialog(this, generator);
		dialog.generate();
		Exception e = dialog.checkException();
		if (e == null) {
			ReplacementMap map = new ReplacementMap();
			map.addReplacement("%FILENAME%", exeFile.getName());
			JOptionPane.showMessageDialog(this, pack.getMessage(
					"MAIN_WINDOW_SUCCESS1_MESSAGE", map), pack
					.getMessage("MAIN_WINDOW_SUCCESS1_TITLE"),
					JOptionPane.INFORMATION_MESSAGE);
		} else {
			ReplacementMap map = new ReplacementMap();
			map.addReplacement("%FILENAME%", exeFile.getName());
			int c = JOptionPane.showConfirmDialog(this, pack.getMessage(
					"MAIN_WINDOW_CONFIRM2_MESSAGE", map), pack
					.getMessage("MAIN_WINDOW_CONFIRM2_TITLE"),
					JOptionPane.ERROR_MESSAGE, JOptionPane.YES_NO_OPTION);
			if (c == JOptionPane.YES_OPTION) {
				ErrorLogDialog logDialog = new ErrorLogDialog(this, e);
				logDialog.setVisible(true);
			}
		}
	}

	/**
	 * Controlla se il progetto corrente  salvato.
	 * 
	 * @return true se il progetto  salvato, false altrimenti.
	 */
	private boolean checkCurrentProjectIsSaved() {
		// Legge le informazioni dai form.
		String applicationName = applicationNameField.getText().trim();
		int applicationType = guiRadioButton.isSelected() ? Project.TYPE_GUI
				: Project.TYPE_CONSOLE;
		String jarPath = jarSelectionField.getText().trim();
		String exePath = exeSelectionField.getText().trim();
		String iconPath = iconSelectionField.getText().trim();
		String jvmParameters = jvmParametersField.getText().trim();
		boolean empty = (applicationName.length() == 0 || jarPath.length() == 0);
		if (empty) {
			return true;
		}
		// C' un progetto corrente?
		if (project == null) {
			return false;
		}
		// Il progetto  stato modificato?
		boolean modified = false;
		modified = modified
				|| !applicationName.equals(project.getApplicationName());
		modified = modified
				|| (applicationType != project.getApplicationType());
		modified = modified || !jarPath.equals(project.getJarPath());
		modified = modified || !exePath.equals(project.getExePath());
		modified = modified || !iconPath.equals(project.getIconPath());
		modified = modified
				|| !jvmParameters.equals(project.getJvmParameters());
		modified = modified || (project.getLocation() == null);
		return !modified;
	}

	/**
	 * Avvia un nuovo progetto.
	 */
	private void newProject() {
		if (checkProjectDiscard()) {
			project = null;
			applicationNameField.setText("");
			guiRadioButton.setSelected(true);
			jarSelectionField.setText("");
			exeSelectionField.setText("");
			iconSelectionField.setText("");
		}
	}

	/**
	 * Carica un progetto da disco.
	 */
	private void openProject() {
		if (checkProjectDiscard()) {
			LanguagePack pack = LanguagePack.getInstance();
			Preferences preferences = Preferences.getInstance();
			JFileChooser chooser = new JFileChooser();
			chooser.setFileFilter(new ProjectFileFilter());
			chooser.setMultiSelectionEnabled(false);
			chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
			chooser.setDialogTitle(pack
					.getMessage("FILECHOOSER_PROJECT_OPEN_TITLE"));
			chooser.setApproveButtonText(pack
					.getMessage("FILECHOOSER_PROJECT_OPEN_APPROVE"));
			String dirPath = preferences.getString(getClass().getName()
					+ ".project.directory");
			if (dirPath != null) {
				chooser.setCurrentDirectory(new File(dirPath));
			} else {
				chooser.setCurrentDirectory(null);
			}
			int c = chooser.showOpenDialog(this);
			if (c == JFileChooser.APPROVE_OPTION) {
				File selectedFile = chooser.getSelectedFile();
				if (selectedFile != null) {
					File selectedDir = selectedFile.getParentFile();
					preferences.putString(getClass().getName()
							+ ".project.directory", selectedDir
							.getAbsolutePath());
					Exception exception = null;
					try {
						project = Project.load(selectedFile);
						project.setLocation(selectedFile);
					} catch (Exception e) {
						exception = e;
					}
					if (exception == null) {
						applicationNameField.setText(project
								.getApplicationName());
						guiRadioButton
								.setSelected(project.getApplicationType() == Project.TYPE_GUI);
						consoleRadioButton.setSelected(project
								.getApplicationType() == Project.TYPE_CONSOLE);
						jarSelectionField.setText(project.getJarPath());
						exeSelectionField.setText(project.getExePath());
						iconSelectionField.setText(project.getIconPath());
						jvmParametersField.setText(project.getJvmParameters());
					} else {
						ReplacementMap map = new ReplacementMap();
						map
								.addReplacement("%FILENAME%", selectedFile
										.getName());
						int k = JOptionPane.showConfirmDialog(this,
								pack.getMessage("MAIN_WINDOW_CONFIRM2_MESSAGE",
										map),
								pack.getMessage("MAIN_WINDOW_CONFIRM2_TITLE"),
								JOptionPane.ERROR_MESSAGE,
								JOptionPane.YES_NO_OPTION);
						if (k == JOptionPane.YES_OPTION) {
							ErrorLogDialog logDialog = new ErrorLogDialog(this,
									exception);
							logDialog.setVisible(true);
						}
					}
				}
			}
		}
	}

	/**
	 * Salva il progetto corrente.
	 */
	private void saveProject(boolean saveAs) {
		LanguagePack pack = LanguagePack.getInstance();
		// Legge le informazioni dai form.
		String applicationName = applicationNameField.getText().trim();
		int applicationType = guiRadioButton.isSelected() ? Project.TYPE_GUI
				: Project.TYPE_CONSOLE;
		String jarPath = jarSelectionField.getText().trim();
		String exePath = exeSelectionField.getText().trim();
		String iconPath = iconSelectionField.getText().trim();
		String jvmParameters = jvmParametersField.getText().trim();
		if (applicationName.length() == 0) {
			JOptionPane.showMessageDialog(this, pack
					.getMessage("MAIN_WINDOW_ERROR1_MESSAGE"), pack
					.getMessage("MAIN_WINDOW_ERROR1_TITLE"),
					JOptionPane.ERROR_MESSAGE);
			return;
		}
		if (jarPath.length() == 0) {
			JOptionPane.showMessageDialog(this, pack
					.getMessage("MAIN_WINDOW_ERROR2_MESSAGE"), pack
					.getMessage("MAIN_WINDOW_ERROR2_TITLE"),
					JOptionPane.ERROR_MESSAGE);
			return;
		}
		Project oldProject = null;
		if (project == null) {
			project = new Project();
		} else {
			try {
				oldProject = (Project) project.clone();
			} catch (CloneNotSupportedException e) {
				;
			}
		}
		project.setApplicationName(applicationName);
		project.setApplicationType(applicationType);
		project.setJarPath(jarPath);
		project.setExePath(exePath);
		project.setIconPath(iconPath);
		project.setJvmParameters(jvmParameters);
		File file = project.getLocation();
		if (file == null || saveAs) {
			Preferences preferences = Preferences.getInstance();
			JFileChooser chooser = new JFileChooser();
			chooser.setFileFilter(new ProjectFileFilter());
			chooser.setMultiSelectionEnabled(false);
			chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
			chooser.setDialogTitle(pack
					.getMessage("FILECHOOSER_PROJECT_SAVE_TITLE"));
			chooser.setApproveButtonText(pack
					.getMessage("FILECHOOSER_PROJECT_SAVE_APPROVE"));
			if (file != null) {
				chooser.setCurrentDirectory(file.getParentFile());
				chooser.setSelectedFile(file);
			} else {
				String dirPath = preferences.getString(getClass().getName()
						+ ".project.directory");
				if (dirPath != null) {
					chooser.setCurrentDirectory(new File(dirPath));
				} else {
					chooser.setCurrentDirectory(null);
				}
			}
			int c = chooser.showOpenDialog(this);
			if (c == JFileChooser.APPROVE_OPTION) {
				File selectedFile = chooser.getSelectedFile();
				if (selectedFile != null) {
					File selectedDir = selectedFile.getParentFile();
					preferences.putString(getClass().getName()
							+ ".project.directory", selectedDir
							.getAbsolutePath());
					String name = selectedFile.getName();
					if (!name.toLowerCase().endsWith(".ksp")) {
						selectedFile = new File(selectedDir, name + ".ksp");
					}
					if (selectedFile.exists()) {
						ReplacementMap map = new ReplacementMap();
						map
								.addReplacement("%FILENAME%", selectedFile
										.getName());
						int res = JOptionPane.showConfirmDialog(this,
								pack.getMessage("MAIN_WINDOW_CONFIRM1_MESSAGE",
										map),
								pack.getMessage("MAIN_WINDOW_CONFIRM1_TITLE"),
								JOptionPane.YES_NO_OPTION);
						if (res != JOptionPane.YES_OPTION) {
							return;
						}
					}
					file = selectedFile;
				}
			}
		}
		if (file != null) {
			Exception exception = null;
			try {
				Project.save(project, file);
				project.setLocation(file);
			} catch (Exception e) {
				project = oldProject;
				exception = e;
			}
			if (exception == null) {
				ReplacementMap map = new ReplacementMap();
				map.addReplacement("%FILENAME%", file.getName());
				JOptionPane.showMessageDialog(this, pack.getMessage(
						"MAIN_WINDOW_SUCCESS2_MESSAGE", map), pack
						.getMessage("MAIN_WINDOW_SUCCESS2_TITLE"),
						JOptionPane.INFORMATION_MESSAGE);
			} else {
				ReplacementMap map = new ReplacementMap();
				map.addReplacement("%FILENAME%", file.getName());
				int c = JOptionPane.showConfirmDialog(this, pack.getMessage(
						"MAIN_WINDOW_CONFIRM2_MESSAGE", map), pack
						.getMessage("MAIN_WINDOW_CONFIRM2_TITLE"),
						JOptionPane.ERROR_MESSAGE, JOptionPane.YES_NO_OPTION);
				if (c == JOptionPane.YES_OPTION) {
					ErrorLogDialog logDialog = new ErrorLogDialog(this,
							exception);
					logDialog.setVisible(true);
				}
			}
		}
	}

	/**
	 * Verifica se il progetto corrente  salvato, e restituisce true se 
	 * possibile abbandonarlo.
	 * 
	 * @return true se  possibile abbandonare il progetto corrente, false
	 *         altrimenti.
	 */
	private boolean checkProjectDiscard() {
		boolean saved = checkCurrentProjectIsSaved();
		if (!saved) {
			LanguagePack pack = LanguagePack.getInstance();
			ReplacementMap map = new ReplacementMap();
			map.addReplacement("%PROJECTNAME%", (project != null) ? project
					.getApplicationName() : applicationNameField.getText()
					.trim());
			int c;
			if (project == null) {
				c = JOptionPane.showConfirmDialog(this, pack.getMessage(
						"MAIN_WINDOW_CONFIRM3_MESSAGE", map), pack
						.getMessage("MAIN_WINDOW_CONFIRM3_TITLE"),
						JOptionPane.YES_NO_CANCEL_OPTION);
			} else {
				c = JOptionPane.showConfirmDialog(this, pack.getMessage(
						"MAIN_WINDOW_CONFIRM4_MESSAGE", map), pack
						.getMessage("MAIN_WINDOW_CONFIRM4_TITLE"),
						JOptionPane.YES_NO_CANCEL_OPTION);
			}
			if (c == JOptionPane.YES_OPTION) {
				saveProject(false);
				return true;
			} else if (c == JOptionPane.NO_OPTION) {
				return true;
			} else {
				return false;
			}
		} else {
			return true;
		}
	}

	/**
	 * Mostra la finestra con le informazioni sul programma.
	 */
	private void showAboutDialog() {
		AboutDialog dialog = new AboutDialog(this);
		dialog.setVisible(true);
	}

	/**
	 * Avvia l'applicazione.
	 */
	public static void startApplication() {
		// Imposta il look and feel.
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			;
		}
		// Apre il frame principale.
		Kickstart kickstart = new Kickstart();
		kickstart.setVisible(true);
	}

}
