package it.sauronsoftware.kickstart;

import it.sauronsoftware.kickstart.util.ReplacementMap;

import java.io.InputStream;
import java.net.URL;
import java.util.Properties;

/**
 * Permette di caricare i messaggi localizzati secondo il sistema operativo
 * dell'utente.
 * 
 * @author Carlo Pelliccia
 */
public class LanguagePack {

	/**
	 * L'istanza per il singleton.
	 */
	private static LanguagePack instance = null;

	/**
	 * Restituisce l'istanza del singleton.
	 * 
	 * @return L'istanza di LanguagePack per l'applicazione.
	 */
	public static LanguagePack getInstance() {
		if (instance == null) {
			// Crea l'istanza.
			instance = new LanguagePack();
			// Carica i messaggi secondo la nazionalit.
			instance.load();
		}
		return instance;
	}

	/**
	 * L'oggetto properties che conserva i messaggi.
	 */
	private Properties messages = new Properties();

	/**
	 * Costruttore, privato per inibirne l'uso dall'esterno.
	 */
	private LanguagePack() {
	}

	/**
	 * Carica i messaggi del pack.
	 */
	private void load() {
		String language = System.getProperty("user.language", "en");
		InputStream stream = null;
		try {
			URL url = getClass().getResource(
					"res/languages/" + language + ".properties");
			if (url == null) {
				url = getClass().getResource("res/languages/en.properties");
			}
			stream = url.openStream();
			messages.load(stream);
		} catch (Exception e) {
			;
		} finally {
			if (stream != null) {
				try {
					stream.close();
				} catch (Throwable t) {
					;
				}
			}
		}
	}

	/**
	 * Restituisce uno dei messaggi del pack.
	 * 
	 * @param key
	 *            La chiave identificativa del messaggio.
	 * @return Il messaggio richiesto.
	 */
	public String getMessage(String key) {
		return messages.getProperty(key, "%" + key + "%");
	}

	/**
	 * Restituisce uno dei messaggi del pack, applicando un set di sostituzioni.
	 * 
	 * @param key
	 *            La chiave identificativa del messaggio.
	 * @param map
	 *            La mappa delle sostituzioni.
	 * @return Il messaggio richiesto.
	 */
	public String getMessage(String key, ReplacementMap map) {
		String message = messages.getProperty(key, "%" + key + "%");
		return map.apply(message);
	}

}
