package it.sauronsoftware.kickstart;

import it.sauronsoftware.kickstart.util.ReplacementMap;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;

/**
 * La classe (singleton) che permette la gestione delle preferenze dell'utente.
 * 
 * @author Carlo Pelliccia
 */
public class Preferences {

	/**
	 * L'istanza per il singleton.
	 */
	private static Preferences instance = null;

	/**
	 * Restituisce l'istanza delle preferenze abbinata all'applicazione.
	 * 
	 * @return Le preferenze abbinate all'applicazione.
	 */
	public static Preferences getInstance() {
		if (instance == null) {
			// Crea l'istanza.
			instance = new Preferences();
			// Carica le preferenze.
			instance.load();
			// Aggiunge uno shutdown hook per il salvataggio delle preferenze.
			Thread thread = new Thread() {
				public void run() {
					instance.save();
				}
			};
			Runtime.getRuntime().addShutdownHook(thread);
		}
		return instance;
	}

	/**
	 * Il file nel quale vengono rese persistenti le preferenze.
	 */
	private File file = new File(System.getProperty("user.home"),
			".kickstart/preferences");

	/**
	 * La tavola con cui vengono mantenute in memoria le preferenze.
	 */
	private Hashtable preferences = new Hashtable();

	/**
	 * La mappa delle sostituzioni da applicare nella codifica e nella
	 * decodifica delle stringhe durante il loro stoccaggio.
	 */
	private ReplacementMap replacements = new ReplacementMap();

	/**
	 * Crea l'istanza. Privato per evitare accesso dall'esterno. Usare come
	 * singeton.
	 */
	private Preferences() {
		// Popola la mappa delle sostituzioni.
		replacements.addReplacement("\\", "\\\\");
		replacements.addReplacement(":", "\\dp");
		replacements.addReplacement("\r", "\\r");
		replacements.addReplacement("\n", "\\n");
	}

	/**
	 * Carica le preferenze dal file di persistenza delle stesse.
	 */
	private void load() {
		BufferedReader reader = null;
		try {
			reader = new BufferedReader(new FileReader(file));
			String line;
			while ((line = reader.readLine()) != null) {
				String[] tokens = line.split(":");
				if (tokens.length == 3) {
					String name = decode(tokens[0]);
					String type = decode(tokens[1]);
					String value = decode(tokens[2]);
					if (type.equals("string")) {
						preferences.put(name, value);
					} else if (type.equals("integer")) {
						try {
							int i = Integer.parseInt(value);
							preferences.put(name, new Integer(i));
						} catch (Exception e) {
							;
						}
					}
				}
			}
		} catch (IOException e) {
			;
		} finally {
			if (reader != null) {
				try {
					reader.close();
				} catch (Throwable t) {
					;
				}
			}
		}
	}

	/**
	 * Salva le preferenze nel file di persistenza.
	 */
	private void save() {
		file.getParentFile().mkdirs();
		BufferedWriter writer = null;
		try {
			writer = new BufferedWriter(new FileWriter(file));
			Enumeration keys = preferences.keys();
			while (keys.hasMoreElements()) {
				String name = (String) keys.nextElement();
				Object value = preferences.get(name);
				String type = null;
				String valueString = null;
				if (value instanceof String) {
					type = "string";
					valueString = (String) value;
				} else if (value instanceof Integer) {
					type = "integer";
					valueString = String.valueOf(((Integer) value).intValue());
				}
				if (type != null) {
					writer.write(encode(name) + ":" + encode(type) + ":"
							+ encode(valueString));
					writer.newLine();
					writer.flush();
				}
			}
		} catch (Exception e) {
			;
		} finally {
			if (writer != null) {
				try {
					writer.close();
				} catch (Throwable t) {
					;
				}
			}
		}
	}

	/**
	 * Decodifica una stringa estratta dal file di persistenza.
	 * 
	 * @param str
	 *            La stringa codificata.
	 * @return La stringa decodificata.
	 */
	private String decode(String str) {
		return replacements.revert(str);
	}

	/**
	 * Codifica una stringa per il file di persistenza.
	 * 
	 * @param str
	 *            La stringa da codificare.
	 * @return La stringa codificata.
	 */
	private String encode(String str) {
		return replacements.apply(str);
	}

	/**
	 * Memorizza una preferenza di tipo stringa. Se gi esiste una preferenza
	 * con il nome specificato, indipendentemente dal tipo, il suo valore viene
	 * sovrascritto.
	 * 
	 * @param name
	 *            Il nome della preferenza.
	 * @param value
	 *            Il valore della preferenza.
	 */
	public void putString(String name, String value) {
		preferences.put(name, value);
	}

	/**
	 * Memorizza una preferenza di tipo intero. Se gi esiste una preferenza con
	 * il nome specificato, indipendentemente dal tipo, il suo valore viene
	 * sovrascritto.
	 * 
	 * @param name
	 *            Il nome della preferenza.
	 * @param value
	 *            Il valore della preferenza.
	 */
	public void putInt(String name, int value) {
		preferences.put(name, new Integer(value));
	}

	/**
	 * Restituisce il valore di una preferenza di tipo stringa. Se la preferenza
	 * richiesta non esiste, o il suo tipo non  stringa, viene restituito il
	 * valore di default.
	 * 
	 * @param name
	 *            Il nome della preferenza.
	 * @param defValue
	 *            Il valore di default da restituire nel caso in cui la
	 *            preferenza non esista o non sia di tipo stringa.
	 * @return Il valore della preferenza, o il valore di default se la
	 *         preferenza non esiste o non  di tipo stringa.
	 */
	public String getString(String name, String defValue) {
		Object obj = preferences.get(name);
		if (obj != null && obj instanceof String) {
			return (String) obj;
		} else {
			return defValue;
		}
	}

	/**
	 * Restituisce il valore di una preferenza di tipo intero. Se la preferenza
	 * richiesta non esiste, o il suo tipo non  intero, viene restituito il
	 * valore di default.
	 * 
	 * @param name
	 *            Il nome della preferenza.
	 * @param defValue
	 *            Il valore di default da restituire nel caso in cui la
	 *            preferenza non esista o non sia di tipo intero.
	 * @return Il valore della preferenza, o il valore di default se la
	 *         preferenza non esiste o non  di tipo intero.
	 */
	public int getInt(String name, int defValue) {
		Object obj = preferences.get(name);
		if (obj != null && obj instanceof Integer) {
			return ((Integer) obj).intValue();
		} else {
			return defValue;
		}
	}

	/**
	 * Restituisce il valore di una preferenza di tipo stringa. Se la preferenza
	 * richiesta non esiste, o il suo tipo non  stringa, viene restituito null.
	 * 
	 * @param name
	 *            Il nome della preferenza.
	 * @return Il valore della preferenza, o null se la preferenza non esiste o
	 *         non  di tipo stringa.
	 */
	public String getString(String name) {
		return getString(name, null);
	}

	/**
	 * Restituisce il valore di una preferenza di tipo intero. Se la preferenza
	 * richiesta non esiste, o il suo tipo non  intero, viene restituito il
	 * valore 0.
	 * 
	 * @param name
	 *            Il nome della preferenza.
	 * @return Il valore della preferenza, o 0 se la preferenza non esiste o non
	 *          di tipo intero.
	 */
	public int getInt(String name) {
		return getInt(name, 0);
	}

}
