package it.sauronsoftware.kickstart.ant;

import it.sauronsoftware.kickstart.generator.EXEGenerator;
import it.sauronsoftware.kickstart.generator.GenerationListener;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;

/**
 * Task di kickstart per ant.
 * 
 * @author Carlo Pelliccia
 */
public class KickstartTask extends Task {

	/**
	 * Il nome dell'applicazione.
	 */
	private String name = null;

	/**
	 * Il tipo dell'applicazione.
	 */
	private int type = EXEGenerator.TYPE_GUI;

	/**
	 * I parametri per la macchina virtuale.
	 */
	private String jvmParameters = "";

	/**
	 * Il file JAR di origine.
	 */
	private String jar = null;

	/**
	 * Il file EXE di destinazione.
	 */
	private String exe = null;

	/**
	 * Il file con l'icona.
	 */
	private String icon = null;

	/**
	 * Imposta il file EXE di destinazione.
	 * 
	 * @param exe
	 *            Il file EXE di destinazione.
	 */
	public void setExe(String exe) {
		this.exe = exe;
	}

	/**
	 * Imposta il file con l'icona.
	 * 
	 * @param icon
	 *            Il file con l'icona.
	 */
	public void setIcon(String icon) {
		this.icon = icon;
	}

	/**
	 * Imposta il file JAR di origine.
	 * 
	 * @param jar
	 *            Il file JAR di origine.
	 */
	public void setJar(String jar) {
		this.jar = jar;
	}

	/**
	 * Imposta i parametri per la macchina virtuale.
	 * 
	 * @param jvmParameters
	 *            I parametri per la macchina virtuale.
	 */
	public void setJvmParameters(String jvmParameters) {
		this.jvmParameters = jvmParameters;
	}

	/**
	 * Imposta il nome dell'applicazione.
	 * 
	 * @param name
	 *            Il nome dell'applicazione.
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Imposta il tipo dell'applicazione.
	 * 
	 * @param type
	 *            Il tipo dell'applicazione.
	 * @throws IllegalArgumentException
	 *             Se il tipo non  "gui" o "console".
	 */
	public void setType(String type) throws IllegalArgumentException {
		if (type.equalsIgnoreCase("gui")) {
			this.type = EXEGenerator.TYPE_GUI;
		} else if (type.equalsIgnoreCase("console")) {
			this.type = EXEGenerator.TYPE_CONSOLE;
		} else {
			throw new IllegalArgumentException("Invalid type: " + type
					+ ". Should be \"gui\" or \"console\".");
		}
	}

	/**
	 * Esegue il task.
	 * 
	 * @throws BuildException
	 *             In caso di fallimento.
	 */
	public void execute() throws BuildException {
		// Controlla la consistenza dei dati.
		if (name == null || (name = name.trim()).length() == 0) {
			throw new BuildException("Application name not specified");
		}
		if (jar == null || (jar = jar.trim()).length() == 0) {
			throw new BuildException("Source JAR file not specified");
		}
		if (exe == null || (exe = exe.trim()).length() == 0) {
			throw new BuildException("Target EXE file not specified");
		}
		File jarFile = new File(jar);
		if (!jarFile.exists()) {
			throw new BuildException("Source JAR file not found at path: "
					+ jarFile.getAbsolutePath());
		}
		File exeFile = new File(exe);
		URL iconURL = null;
		if (icon != null && (icon = icon.trim()).length() > 0) {
			File iconFile = new File(icon);
			if (!iconFile.exists()) {
				throw new BuildException("Icon file not found at path: "
						+ iconFile.getAbsolutePath());
			}
			try {
				iconURL = iconFile.toURI().toURL();
			} catch (MalformedURLException e) {
				;
			}
		}
		System.out.println("Generating EXE: " + exeFile.getAbsolutePath());
		final ExceptionWrapper wrapper = new ExceptionWrapper();
		EXEGenerator generator = new EXEGenerator(name, type, jvmParameters,
				jarFile, exeFile, iconURL);
		generator.setGenerationListener(new GenerationListener() {
			public void failed(Exception e) {
				wrapper.exception = e;
			}

			public void completed() {
				;
			}

			public void percentual(int value) {
				;
			}

			public void started() {
				;
			}
		});
		generator.run();
		if (wrapper.exception != null) {
			throw new BuildException(wrapper.exception);
		}
	}

	/**
	 * Banale struttura dati per il wrapping di un'eccezione.
	 */
	private static class ExceptionWrapper {

		public Exception exception = null;

	}

}
