package it.sauronsoftware.kickstart.generator;

import it.sauronsoftware.kickstart.util.ReplacementMap;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.net.URL;

/**
 * La routine di generazione di un EXE.
 * 
 * @author Carlo Pelliccia
 */
public class EXEGenerator implements Runnable {

	/**
	 * Costante per il tipo "applicazione a finestre".
	 */
	public static final int TYPE_GUI = 0;

	/**
	 * Costante per il tipo "applicazione per console".
	 */
	public static final int TYPE_CONSOLE = 1;

	/**
	 * Il nome dell'applicazione.
	 */
	private String applicationName = null;

	/**
	 * Il tipo dell'applicazione.
	 */
	private int applicationType = TYPE_GUI;

	/**
	 * I parametri aggiuntivi per la macchina virtuale.
	 */
	private String vmparameters = "";

	/**
	 * Il file JAR di origine.
	 */
	private File jarFile = null;

	/**
	 * Il file EXE di destinazione.
	 */
	private File exeFile = null;

	/**
	 * Il file con l'icona.
	 */
	private URL iconURL = null;

	/**
	 * Il listener sul progresso dell'operazione.
	 */
	private GenerationListener listener = null;

	/**
	 * Prepara la routine.
	 * 
	 * @param applicationName
	 *            Il nome dell'applicazione.
	 * @param applicationType
	 *            Il tipo dell'applicazione.
	 * @param vmparameters
	 *            I parametri aggiuntivi per la macchina virtuale.
	 * @param jarFile
	 *            Il file JAR di origine.
	 * @param exeFile
	 *            Il file EXE di destinazione.
	 * @param iconURL
	 *            Il file con l'icona.
	 */
	public EXEGenerator(String applicationName, int applicationType,
			String vmparameters, File jarFile, File exeFile, URL iconURL) {
		this.applicationName = applicationName;
		this.applicationType = applicationType;
		this.vmparameters = vmparameters;
		this.jarFile = jarFile;
		this.exeFile = exeFile;
		this.iconURL = iconURL;
	}

	/**
	 * Imposta il listener sul progresso dell'operazione.
	 * 
	 * @param listener
	 *            Il listener sul progresso dell'operazione.
	 */
	public void setGenerationListener(GenerationListener listener) {
		this.listener = listener;
	}

	/**
	 * La routine di generazione dell'EXE.
	 */
	public void run() {
		int percentual = 0;
		if (listener != null) {
			listener.started();
		}
		ReplacementMap escapeMap = new ReplacementMap();
		escapeMap.addReplacement("'", "$\\'");
		ReplacementMap map = new ReplacementMap();
		map.addReplacement("[APPNAME]", escapeMap.apply(applicationName));
		map.addReplacement("[TYPE]", (applicationType == TYPE_CONSOLE) ? "java"
				: "javaw");
		map.addReplacement("[VMP]", escapeMap.apply(vmparameters));
		File tempDir = new File(System.getProperty("java.io.tmpdir"),
				".kickstart");
		tempDir.mkdirs();
		try {
			File makensis = new File(tempDir, "makensis.exe");
			URL aux = getClass().getResource("files/makensis.exe");
			saveStream(aux.openStream(), makensis);
			if (listener != null) {
				listener.percentual(percentual += 15);
			}
			File iconFile2 = new File(tempDir, "icon.ico");
			if (iconURL == null) {
				aux = getClass().getResource("files/icon.ico");
				saveStream(aux.openStream(), iconFile2);
			} else {
				saveStream(iconURL.openStream(), iconFile2);
			}
			if (listener != null) {
				listener.percentual(percentual += 15);
			}
			File sourceFile2 = new File(tempDir, "source.jar");
			copyFile(jarFile, sourceFile2);
			if (listener != null) {
				listener.percentual(percentual += 15);
			}
			File script = new File(tempDir, "script.nsi");
			BufferedReader reader = null;
			BufferedWriter writer = null;
			try {
				aux = getClass().getResource("files/script.nsi");
				reader = new BufferedReader(new InputStreamReader(aux
						.openStream()));
				writer = new BufferedWriter(new OutputStreamWriter(
						new FileOutputStream(script)));
				String line;
				while ((line = reader.readLine()) != null) {
					line = map.apply(line);
					writer.write(line + "\r\n");
				}
			} catch (Exception e) {
				throw e;
			} finally {
				if (writer != null)
					try {
						writer.flush();
						writer.close();
					} catch (Exception e) {
					}
				if (reader != null)
					try {
						reader.close();
					} catch (Exception e) {
					}
			}
			if (listener != null) {
				listener.percentual(percentual += 15);
			}
			Process p = Runtime.getRuntime().exec(
					"\"" + makensis.getAbsolutePath() + "\" \""
							+ script.getAbsolutePath() + "\"");
			BufferedReader preader = new BufferedReader(new InputStreamReader(p
					.getInputStream()));
			while (preader.readLine() != null) {
				if (percentual < 90) {
					listener.percentual(percentual += 1);
				}
			}
			int i = p.exitValue();
			if (i != 0) {
				throw new Exception();
			}
			if (listener != null) {
				listener.percentual(percentual = 90);
			}
			File temp = new File(tempDir, "destination.exe");
			copyFile(temp, exeFile);
			if (listener != null) {
				listener.percentual(percentual = 100);
				listener.completed();
			}
		} catch (Exception e) {
			if (listener != null) {
				listener.failed(e);
			}
		} finally {
			tempDir.delete();
		}
	}

	/**
	 * Copia un file.
	 * 
	 * @param f1
	 *            Il file di origine.
	 * @param f2
	 *            Il file di destinazione.
	 * @throws Exception
	 *             In caso di errore.
	 */
	private void copyFile(File f1, File f2) throws Exception {
		InputStream in = null;
		try {
			in = new FileInputStream(f1);
			saveStream(in, f2);
		} catch (Exception e) {
			throw e;
		} finally {
			if (in != null)
				try {
					in.close();
				} catch (Exception e) {
				}
		}
	}

	/**
	 * Copia uno stream in un file.
	 * 
	 * @param in
	 *            Lo stream di origine.
	 * @param f2
	 *            Il file di destinazione.
	 * @throws Exception
	 *             In caso di errore.
	 */
	private void saveStream(InputStream in, File f2) throws Exception {
		OutputStream out = null;
		try {
			out = new FileOutputStream(f2);
			byte[] buffer = new byte[1024];
			int l;
			while ((l = in.read(buffer, 0, buffer.length)) != -1) {
				out.write(buffer, 0, l);
			}
		} catch (Exception e) {
			throw e;
		} finally {
			if (out != null)
				try {
					out.close();
				} catch (Exception e) {
				}
			if (in != null)
				try {
					in.close();
				} catch (Exception e) {
				}
		}
	}

}
