package it.sauronsoftware.kickstart.project;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Date;
import java.util.Properties;

/**
 * Rappresenta i progetti di kickstart.
 * 
 * @author Carlo Pelliccia
 */
public class Project implements Cloneable {

	/**
	 * Costante per il tipo "applicazione a finestre".
	 */
	public static final int TYPE_GUI = 0;

	/**
	 * Costante per il tipo "applicazione per console".
	 */
	public static final int TYPE_CONSOLE = 1;

	/**
	 * Il percorso dove  salvato il progetto.
	 */
	private File location = null;

	/**
	 * Il nome dell'applicazione.
	 */
	private String applicationName = null;

	/**
	 * Il tipo dell'applicazione.
	 */
	private int applicationType = TYPE_GUI;

	/**
	 * I parametri per la macchina virtuale.
	 */
	private String jvmParameters = null;

	/**
	 * Il percorso del file JAR di origine.
	 */
	private String jarPath = null;

	/**
	 * Il percorso del file EXE di destinazione.
	 */
	private String exePath = null;

	/**
	 * Il percorso del file con l'icona.
	 */
	private String iconPath = null;

	/**
	 * Restituisce il nome dell'applicazione.
	 * 
	 * @return Il nome dell'applicazione.
	 */
	public String getApplicationName() {
		return applicationName;
	}

	/**
	 * Imposta il nome dell'applicazione.
	 * 
	 * @param applicationName
	 *            Il nome dell'applicazione.
	 */
	public void setApplicationName(String applicationName) {
		this.applicationName = applicationName;
	}

	/**
	 * Restituisce il tipo dell'applicazione.
	 * 
	 * @return Il tipo dell'applicazione.
	 */
	public int getApplicationType() {
		return applicationType;
	}

	/**
	 * Imposta il tipo dell'applicazione.
	 * 
	 * @param applicationType
	 *            Il tipo dell'applicazione.
	 */
	public void setApplicationType(int applicationType) {
		this.applicationType = applicationType;
	}

	/**
	 * Restituisce i parametri per la macchina virtuale.
	 * 
	 * @return I parametri per la macchina virtuale.
	 */
	public String getJvmParameters() {
		return jvmParameters;
	}

	/**
	 * Imposta i parametri per la macchina virtuale.
	 * 
	 * @param jvmParameters
	 *            I parametri per la macchina virtuale.
	 */
	public void setJvmParameters(String jvmParameters) {
		this.jvmParameters = jvmParameters;
	}

	/**
	 * Restituisce il percorso del file EXE di destinazione.
	 * 
	 * @return Il percorso del file EXE di destinazione.
	 */
	public String getExePath() {
		return exePath;
	}

	/**
	 * Imposta il percorso del file EXE di destinazione.
	 * 
	 * @param exePath
	 *            Il percorso del file EXE di destinazione.
	 */
	public void setExePath(String exePath) {
		this.exePath = exePath;
	}

	/**
	 * Restituisce il percorso del file con l'icona.
	 * 
	 * @return Il percorso del file con l'icona.
	 */
	public String getIconPath() {
		return iconPath;
	}

	/**
	 * Imposta il percorso del file con l'icona.
	 * 
	 * @param iconPath
	 *            Il percorso del file con l'icona.
	 */
	public void setIconPath(String iconPath) {
		this.iconPath = iconPath;
	}

	/**
	 * Restituisce il percorso del file JAR di origine.
	 * 
	 * @return Il percorso del file JAR di origine.
	 */
	public String getJarPath() {
		return jarPath;
	}

	/**
	 * Imposta il percorso del file JAR di origine.
	 * 
	 * @param jarPath
	 *            Il percorso del file JAR di origine.
	 */
	public void setJarPath(String jarPath) {
		this.jarPath = jarPath;
	}

	/**
	 * Restituisce il percorso dove  salvato il progetto.
	 * 
	 * @return Il percorso dove  salvato il progetto.
	 */
	public File getLocation() {
		return location;
	}

	/**
	 * Imposta il percorso dove  salvato il progetto.
	 * 
	 * @param location
	 *            Il percorso dove  salvato il progetto.
	 */
	public void setLocation(File location) {
		this.location = location;
	}

	public Object clone() throws CloneNotSupportedException {
		Project aux = new Project();
		aux.applicationName = applicationName;
		aux.applicationType = applicationType;
		aux.jvmParameters = jvmParameters;
		aux.exePath = exePath;
		aux.jarPath = jarPath;
		aux.iconPath = iconPath;
		aux.location = location;
		return aux;
	}

	/**
	 * Salva un progetto su file.
	 * 
	 * @param project
	 *            Il progetto dal salvare.
	 * @param file
	 *            Il file su cui salvare il progetto.
	 * @throws Exception
	 *             In caso di errore durante il salvataggio.
	 */
	public static void save(Project project, File file) throws Exception {
		Properties properties = new Properties();
		properties.setProperty("name", project.applicationName);
		properties.setProperty("type", String.valueOf(project.applicationType));
		properties.setProperty("jvmp", project.jvmParameters);
		properties.setProperty("jar", project.jarPath);
		properties.setProperty("exe", project.exePath);
		properties.setProperty("icon", project.iconPath);
		OutputStream stream = null;
		try {
			stream = new FileOutputStream(file);
			properties.store(stream, "Kickstart 2.0 Project, generated on "
					+ new Date());
		} catch (Exception e) {
			throw e;
		} finally {
			if (stream != null) {
				try {
					stream.close();
				} catch (Throwable t) {
					;
				}
			}
		}
	}

	/**
	 * Carica un progetto da file.
	 * 
	 * @param file
	 *            Il file da cui caricare il progetto.
	 * @return In caso di errore.
	 * @throws Exception
	 */
	public static Project load(File file) throws Exception {
		Properties properties = new Properties();
		InputStream stream = null;
		try {
			stream = new FileInputStream(file);
			properties.load(stream);
		} catch (Exception e) {
			throw e;
		} finally {
			if (stream != null) {
				try {
					stream.close();
				} catch (Throwable t) {
					;
				}
			}
		}
		int type = 0;
		try {
			type = Integer.parseInt(properties.getProperty("type", ""));
		} catch (Exception e) {
			;
		}
		Project project = new Project();
		project.applicationName = properties.getProperty("name", "");
		switch (type) {
		case TYPE_GUI:
			project.applicationType = TYPE_GUI;
			break;
		case TYPE_CONSOLE:
			project.applicationType = TYPE_CONSOLE;
			break;
		default:
			project.applicationType = TYPE_GUI;
			break;
		}
		project.jvmParameters = properties.getProperty("jvmp", "");
		project.jarPath = properties.getProperty("jar", "");
		project.exePath = properties.getProperty("exe", "");
		project.iconPath = properties.getProperty("icon", "");
		return project;
	}

}
